@@ -10,8 +10,8 @@
 from __future__ import print_function
 import requests
 import json
-from watson_machine_learning_client.utils import DEPLOYMENT_DETAILS_TYPE, INSTANCE_DETAILS_TYPE, print_text_header_h1, print_text_header_h2, STR_TYPE, STR_TYPE_NAME, docstring_parameter, str_type_conv, meta_props_str_conv, StatusLogger, convert_metadata_to_parameters, create_download_link, is_ipython
-from watson_machine_learning_client.wml_client_error import WMLClientError, MissingValue, ApiRequestFailure
+from watson_machine_learning_client.utils import DEPLOYMENT_DETAILS_TYPE, INSTANCE_DETAILS_TYPE, print_text_header_h1, print_text_header_h2, STR_TYPE, STR_TYPE_NAME, docstring_parameter, str_type_conv, meta_props_str_conv, StatusLogger, convert_metadata_to_parameters
+from watson_machine_learning_client.wml_client_error import WMLClientError, MissingValue
 from watson_machine_learning_client.href_definitions import is_uid
 from watson_machine_learning_client.wml_resource import WMLResource
 from watson_machine_learning_client.metanames import PayloadLoggingMetaNames
@@ -90,21 +90,27 @@ class Deployments(WMLResource):
         response = None
 
         if name is not None or description is not None:
-            meta_props_names = {
-                "name": name,
-                "description": description
-            }
-
-            meta_props_data = []
+            patch_payload = []
 
             if name is not None:
-                meta_props_data.append({"name": "name", "type": STR_TYPE, "path": ["name"]})
+                patch_payload.append({
+                    'op': 'replace',
+                    'path': '/name',
+                    'value': name
+                })
 
             if description is not None:
-                meta_props_data.append({"name": "description", "type": STR_TYPE, "path": ["description"]})
+                patch_payload.append({
+                    'op': 'replace',
+                    'path': '/description',
+                    'value': description
+                })
 
             url = self._href_definitions.get_deployment_href(deployment_uid)
-            self._update(url, deployment_details, meta_props_names, meta_props_data)
+            response = requests.patch(url, json=patch_payload, headers=self._client._get_headers())
+            updated_details = self._handle_response(200, u'deployment patch', response)
+
+            return updated_details
 
         deployment_type = str(self._get_required_element_from_dict(deployment_details, 'deployment_details', ['entity', 'type'])).lower()
 
@@ -679,30 +685,33 @@ class Deployments(WMLResource):
 
     def setup_payload_logging(self, deployment_uid, meta_props):
         """
-         Configure payload logging for deployment.
+        .. note::
+            **Closed Beta:** This feature is available to closed beta participants only. There is no backward compatibility warranted after the beta program.
 
-         :param deployment_uid: uid of deployment
-         :type deployment_uid: {str_type}
+        Configure payload logging for deployment.
 
-         :param meta_props: meta data of the payload configuration. To see available meta names use:
+        :param deployment_uid: uid of deployment
+        :type deployment_uid: {str_type}
 
-             >>> client.deployments.PayloadLoggingMetaNames.show()
+        :param meta_props: meta data of the payload configuration. To see available meta names use:
 
-         :type meta_props: dict
+            >>> client.deployments.PayloadLoggingMetaNames.show()
 
-         :returns: payload logging configuration details
-         :rtype: dict
+        :type meta_props: dict
 
-         A way you might use me is:
+        :returns: payload logging configuration details
+        :rtype: dict
 
-         >>> metadata = {
-         >>>  client.training.PayloadLoggingMetaNames.PAYLOAD_DATA_REFERENCE: {},
-         >>>  client.training.PayloadLoggingMetaNames.LABELS: ['a', 'b', 'c'],
-         >>>  client.training.PayloadLoggingMetaNames.OUTPUT_DATA_SCHEMA: {}
-         >>> }
-         >>>
-         >>> config_details = client.deployments.setup_payload_logging(deployment_uid, meta_props=metadata)
-         """
+        A way you might use me is:
+
+        >>> metadata = {
+        >>>  client.training.PayloadLoggingMetaNames.PAYLOAD_DATA_REFERENCE: {},
+        >>>  client.training.PayloadLoggingMetaNames.LABELS: ['a', 'b', 'c'],
+        >>>  client.training.PayloadLoggingMetaNames.OUTPUT_DATA_SCHEMA: {}
+        >>> }
+        >>>
+        >>> config_details = client.deployments.setup_payload_logging(deployment_uid, meta_props=metadata)
+        """
 
         self._validate_if_payload_logging_is_available('setup payload logging')
 
@@ -728,17 +737,20 @@ class Deployments(WMLResource):
 
     def get_payload_logging_details(self, deployment_uid):
         """
-         Get details of payload logging configuration.
+        .. note::
+            **Closed Beta:** This feature is available to closed beta participants only. There is no backward compatibility warranted after the beta program.
 
-         :param deployment_uid: uid of deployment
-         :type deployment_uid: {str_type}
+        Get details of payload logging configuration.
 
-         :returns: payload logging configuration details
-         :rtype: dict
+        :param deployment_uid: uid of deployment
+        :type deployment_uid: {str_type}
 
-         A way you might use me is:
+        :returns: payload logging configuration details
+        :rtype: dict
 
-         >>> config_details = client.deployments.get_payload_logging_details(deployment_uid)
+        A way you might use me is:
+
+        >>> config_details = client.deployments.get_payload_logging_details(deployment_uid)
         """
         self._validate_if_payload_logging_is_available('get details of payload logging')
 
@@ -757,14 +769,17 @@ class Deployments(WMLResource):
 
     def delete_payload_logging(self, deployment_uid):
         """
-          Delete payload logging configuration and stop logging.
+        .. note::
+            **Closed Beta:** This feature is available to closed beta participants only. There is no backward compatibility warranted after the beta program.
 
-          :param deployment_uid: uid of deployment
-          :type deployment_uid: {str_type}
+        Delete payload logging configuration and stop logging.
 
-          A way you might use me is:
+        :param deployment_uid: uid of deployment
+        :type deployment_uid: {str_type}
 
-          >>> client.deployments.delete_payload_logging(deployment_uid)
+        A way you might use me is:
+
+        >>> client.deployments.delete_payload_logging(deployment_uid)
         """
         self._validate_if_payload_logging_is_available('delete payload logging')
 
