@@ -12,12 +12,14 @@ import requests
 import json
 import re
 import time
-from watson_machine_learning_client.wml_client_error import MissingValue, WMLClientError
-from watson_machine_learning_client.href_definitions import is_uid, is_url
+import copy
+from watson_machine_learning_client.wml_client_error import MissingValue, WMLClientError, MissingMetaProp
+from watson_machine_learning_client.href_definitions import is_uid
 from watson_machine_learning_client.wml_resource import WMLResource
 from multiprocessing import Pool
-from watson_machine_learning_client.utils import print_text_header_h1, print_text_header_h2, EXPERIMENT_RUN_DETAILS_TYPE, format_metrics, STR_TYPE, STR_TYPE_NAME, docstring_parameter, group_metrics, str_type_conv
+from watson_machine_learning_client.utils import print_text_header_h1, print_text_header_h2, EXPERIMENT_DETAILS_TYPE, EXPERIMENT_RUN_DETAILS_TYPE, format_metrics, STR_TYPE, STR_TYPE_NAME, docstring_parameter, group_metrics, str_type_conv, meta_props_str_conv
 from watson_machine_learning_client.hpo import HPOParameter, HPOMethodParam
+from watson_machine_learning_client.metanames import ExperimentMetaNames
 
 
 def _get_details_helper(url, headers, setting=None):
@@ -50,6 +52,9 @@ class Experiments(WMLResource):
        Run new experiment.
     """
 
+    ConfigurationMetaNames = ExperimentMetaNames()
+    """MetaNames for experiments creation."""
+
     @staticmethod
     def HPOParameter(name, values=None, max=None, min=None, step=None):
         return HPOParameter(name, values, max, min, step)
@@ -62,6 +67,94 @@ class Experiments(WMLResource):
         WMLResource.__init__(self, __name__, client)
         self._experiments_uids_cache = {}
 
+    def _store(self, meta_props):
+        """
+           Store experiment.
+
+            :param meta_props: meta data of the experiment configuration. To see available meta names use:
+
+               >>> client.experiments.ConfigurationMetaNames.get()
+            :type meta_props: dict
+
+            :returns: stored experiment details
+            :rtype: dict
+
+           A way you might use me is:
+
+           >>> metadata = {
+           >>>  client.experiments.ConfigurationMetaNames.NAME: 'my_experiment',
+           >>>  client.experiments.ConfigurationMetaNames.EVALUATION_METRICS: ['accuracy'],
+           >>>  client.experiments.ConfigurationMetaNames.TRAINING_DATA_REFERENCE: {'connection': {'endpoint_url': 'https://s3-api.us-geo.objectstorage.softlayer.net', 'access_key_id': '***', 'secret_access_key': '***'}, 'source': {'bucket': 'train-data'}, 'type': 's3'},
+           >>>  client.experiments.ConfigurationMetaNames.TRAINING_RESULTS_REFERENCE: {'connection': {'endpoint_url': 'https://s3-api.us-geo.objectstorage.softlayer.net', 'access_key_id': '***', 'secret_access_key': '***'}, 'target': {'bucket': 'result-data'}, 'type': 's3'},
+           >>>  client.experiments.ConfigurationMetaNames.TRAINING_REFERENCES: [
+           >>>      {
+           >>>        'training_definition_url': definition_url_1
+           >>>      },
+           >>>      {
+           >>>        'training_definition_url': definition_url_2
+           >>>      },
+           >>>   ],
+           >>> }
+           >>> experiment_details = client.experiments._store(meta_props=metadata)
+           >>> experiment_url = client.experiments.get_url(experiment_details)
+        """
+        Experiments._validate_type(meta_props, u'meta_props', dict, True)
+        meta_props_str_conv(meta_props)
+        self.ConfigurationMetaNames._validate(meta_props)
+
+        meta_props = copy.deepcopy(meta_props)
+
+        if any(u'training_definition_url' not in x for x in meta_props[self.ConfigurationMetaNames.TRAINING_REFERENCES]):
+            raise MissingMetaProp(u'training_references.training_definition_url')
+
+        for ref in meta_props[self.ConfigurationMetaNames.TRAINING_REFERENCES]:
+            if u'name' not in ref or u'command' not in ref:
+
+                training_definition_response = requests.get(ref[u'training_definition_url'].replace(u'/content', u''), headers=self._client._get_headers())
+                result = self._handle_response(200, u'getting training definition', training_definition_response)
+
+                if not u'name' in ref:
+                    ref.update({u'name': result[u'entity'][u'name']})
+                if not u'command' in ref:
+                    ref.update({u'command': result[u'entity'][u'command']})
+
+        response_experiment_post = requests.post(
+            self._href_definitions.get_experiments_href(),
+            json=self.ConfigurationMetaNames._generate_resource_metadata(meta_props),
+            headers=self._client._get_headers()
+        )
+
+        return self._handle_response(201, u'saving experiment', response_experiment_post)
+
+    @docstring_parameter({'str_type': STR_TYPE_NAME})
+    def _update_experiment(self, experiment_uid, changes):
+        """
+        Updates existing experiment metadata.
+
+        :param experiment_uid: UID of experiment which definition should be updated
+        :type experiment_uid: {str_type}
+
+        :param changes: elements which should be changed, where keys are ConfigurationMetaNames
+        :type changes: dict
+
+        :return: metadata of updated experiment
+        :rtype: dict
+        """
+        experiment_uid = str_type_conv(experiment_uid)
+        self._validate_type(experiment_uid, u'experiment_uid', STR_TYPE, True)
+        self._validate_type(changes, u'changes', dict, True)
+        meta_props_str_conv(changes)
+
+        details = self._client.repository.get_experiment_details(experiment_uid)
+
+        patch_payload = self.ConfigurationMetaNames._generate_patch_payload(details['entity'], changes, with_validation=True)
+
+        url = self._href_definitions.get_experiment_href(experiment_uid)
+        response = requests.patch(url, json=patch_payload, headers=self._client._get_headers())
+        updated_details = self._handle_response(200, u'experiment patch', response)
+
+        return updated_details
+
     def _get_experiment_uid(self, experiment_run_uid=None, experiment_run_url=None):
         if experiment_run_uid is None and experiment_run_url is None:
             raise MissingValue(u'experiment_run_id/experiment_run_url')
@@ -263,13 +356,16 @@ class Experiments(WMLResource):
         return response
 
     @docstring_parameter({'str_type': STR_TYPE_NAME})
-    def get_details(self, experiment_uid=None):
+    def get_details(self, experiment_uid=None, limit=None):
         """
            Get metadata of experiment(s) run(s). If no experiment_uid is provided, runs will be listed for all existing experiments.
 
            :param experiment_uid:  experiment UID (optional)
            :type experiment_uid: {str_type}
 
+           :param limit: limit number of fetched records (optional)
+            :type limit: int
+
            :returns: experiment(s) run(s) metadata
            :rtype: dict
 
@@ -281,6 +377,50 @@ class Experiments(WMLResource):
         experiment_uid = str_type_conv(experiment_uid)
         return self._get_extended_details(experiment_uid, False)
 
+    @staticmethod
+    @docstring_parameter({'str_type': STR_TYPE_NAME})
+    def _get_uid(experiment_details):
+        """
+            Get uid of stored experiment.
+
+            :param experiment_details: stored experiment details
+            :type experiment_details: dict
+
+            :returns: uid of stored experiment
+            :rtype: {str_type}
+
+            A way you might use me is:
+
+            >>> experiment_uid = client.experiments.get_uid(experiment_details)
+        """
+        Experiments._validate_type(experiment_details, u'experiment_details', object, True)
+        Experiments._validate_type_of_details(experiment_details, EXPERIMENT_DETAILS_TYPE)
+
+        return WMLResource._get_required_element_from_dict(experiment_details, u'experiment_details',
+                                                           [u'metadata', u'guid'])
+
+    @staticmethod
+    @docstring_parameter({'str_type': STR_TYPE_NAME})
+    def _get_url(experiment_details):
+        """
+            Get url of stored experiment.
+
+            :param experiment_details:  stored experiment details
+            :type experiment_details: dict
+
+            :returns: url of stored experiment
+            :rtype: {str_type}
+
+            A way you might use me is:
+
+            >>> experiment_url = client.experiments.get_url(experiment_details)
+        """
+        Experiments._validate_type(experiment_details, u'experiment_details', object, True)
+        Experiments._validate_type_of_details(experiment_details, EXPERIMENT_DETAILS_TYPE)
+
+        return WMLResource._get_required_element_from_dict(experiment_details, u'experiment_details',
+                                                           [u'metadata', u'url'])
+
     @staticmethod
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_run_url(experiment_run_details):
