@@ -76,6 +76,17 @@ class LearningSystem(WMLResource):
         meta_props_str_conv(meta_props)
         self.ConfigurationMetaNames._validate(meta_props)
 
+        model_details_response = requests.get(
+            self._wml_credentials['url'] + '/v3/ml_assets/models/{}'.format(model_uid),
+            headers=self._client._get_headers()
+        )
+
+        model_details = self._handle_response(200, u'creating new model', model_details_response)
+        training_data_reference = model_details['entity']['training_data_reference']
+        training_result_reference = model_details['entity']['model_version']['content_location']
+        execution = {}
+
+        print("Model details: {}\n".format(model_details))
         spark_instance = {
             u'credentials': meta_props[self.ConfigurationMetaNames.SPARK_REFERENCE],
             u'version': u'2.1'
@@ -117,17 +128,17 @@ class LearningSystem(WMLResource):
         """
         model_uid = str_type_conv(model_uid)
         self._validate_type(model_uid, u'model_uid', STR_TYPE, True)
+        self._validate_type(changes, u'changes', dict, True)
+        meta_props_str_conv(changes)
 
         details = self.get_details(model_uid)
 
+        patch_payload = self.ConfigurationMetaNames._generate_patch_payload(details, changes,
+                                                                            with_validation=True)
+
         url = self._href_definitions.get_learning_configuration_href(model_uid)
-        updated_details = self._update(url, details, changes, [
-            {"name": self.ConfigurationMetaNames.FEEDBACK_DATA_REFERENCE, "type": dict,     "path": [u'feedback_data_reference']},
-            {"name": self.ConfigurationMetaNames.MIN_FEEDBACK_DATA_SIZE,  "type": int,      "path": [u'min_feedback_data_size']},
-            {"name": self.ConfigurationMetaNames.SPARK_REFERENCE,         "type": dict,     "path": [u'spark_service', u'credentials']},
-            {"name": self.ConfigurationMetaNames.AUTO_RETRAIN,            "type": STR_TYPE, "path": [u'auto_retrain']},
-            {"name": self.ConfigurationMetaNames.AUTO_REDEPLOY,           "type": STR_TYPE, "path": [u'auto_redeploy']}
-        ])
+        response = requests.patch(url, json=patch_payload, headers=self._client._get_headers())
+        updated_details = self._handle_response(200, u'learning system patch', response)
 
         return updated_details
 
