@@ -20,7 +20,8 @@ from .xgboost_model_artifact import XGBoostModelArtifact
 from .wml_experiment_artifact import WmlExperimentArtifact
 from .wml_libraries_artifact import WmlLibrariesArtifact
 from .wml_runtimes_artifact import WmlRuntimesArtifact
+from .hybrid_model_artifact import  HybridModelArtifact
 
 __all__ = ['Artifact', 'ArtifactReader', 'MetaNames', 'MetaProps', 'WmlExperimentArtifact',
            'ModelArtifact', 'PipelineArtifact', 'ScikitModelArtifact', 'XGBoostModelArtifact',
-           'WmlLibrariesArtifact', 'WmlRuntimesArtifact']
+           'WmlLibrariesArtifact', 'WmlRuntimesArtifact', 'HybridModelArtifact']
