@@ -20,6 +20,7 @@ from repository_v3.mlrepositoryartifact.libraries_artifact import LibrariesArtif
 from repository_v3.mlrepositoryartifact.runtimes_artifact import RuntimesArtifact
 from repository_v3.mlrepositoryartifact.generic_archive_pipeline_model_artifact import GenericArchivePipelineModelArtifact
 from repository_v3.mlrepositoryartifact.function_artifact import FunctionArtifact
+from repository_v3.mlrepositoryartifact.hybrid_pipeline_model_artifact import HybridPipelineModelArtifact
 
 if lib_checker.installed_libs[PYSPARK]:
     from .spark_pipeline_model_artifact import SparkPipelineModelArtifact
@@ -131,7 +132,19 @@ def MLRepositoryArtifact(ml_artifact= None,
         return _get_runtimes_artifact(ml_artifact, name, meta_props)
 
     if ml_artifact is None:
-        return _get_experiment_artifact(meta_props=meta_props)
+        if (MetaNames.FRAMEWORK_NAME in meta_props.get()) or (MetaNames.FRAMEWORK_VERSION in meta_props.get()):
+            if meta_props.get()[MetaNames.FRAMEWORK_NAME] == HYBRID:
+               if (MetaNames.CATEGORY in meta_props.get()) or (MetaNames.CATEGORY in meta_props.get()):
+                   if (MetaNames.CONTENT_LOCATION in meta_props.get()) or (MetaNames.CONTENT_LOCATION in meta_props.get()):
+                      return  _get_hybrid_pipeline_model_artifact(name=name, meta_props=meta_props)
+                   else:
+                       raise ValueError("Missing required meta_props: CONTENT_LOCATION for Hybrid model")
+               else:
+                   raise ValueError("Missing required meta_props: CATEGORY for Hybrid model")
+            else:
+                return _get_experiment_artifact(meta_props=meta_props)
+        else:
+            return _get_experiment_artifact(meta_props=meta_props)
 
     if training_dataref is not None:
         json_training_dataref = json.loads(training_dataref)
@@ -220,7 +233,7 @@ def MLRepositoryArtifact(ml_artifact= None,
     # check if ml_artifact is a tar.gz of a tensorflow model
     if isinstance(ml_artifact, str):
 
-        if (MetaNames.FRAMEWORK_VERSION in meta_props.get()) or (MetaNames.FRAMEWORK_VERSION in meta_props.get()):
+        if (MetaNames.FRAMEWORK_NAME in meta_props.get()) or (MetaNames.FRAMEWORK_VERSION in meta_props.get()):
          if meta_props.get()[MetaNames.FRAMEWORK_NAME] == TENSORFLOW:
             if ("tar.gz" not in ml_artifact) and (".tgz" not in ml_artifact):
                 raise TypeError('The artifact specified ( {} ) is not of type "tar.gz" or "tgz". Only compressed archive of '
@@ -264,6 +277,8 @@ def _get_pipeline(pipeline, name, meta_props):
 def _get_experiment_artifact(meta_props):
     return ExperimentArtifact(meta_props=meta_props)
 
+def _get_hybrid_pipeline_model_artifact(name, meta_props):
+    return HybridPipelineModelArtifact(name=name, meta_props=meta_props)
 
 def _get_pipeline_model(pipeline_model, training_data, pipeline_artifact, name, meta_props):
     return SparkPipelineModelArtifact(pipeline_model, training_data=training_data, pipeline_artifact=pipeline_artifact, name=name, meta_props=meta_props)
