@@ -210,6 +210,8 @@ class ModelCollection:
                return self._save_generic_archive_pipeline_model(artifact, query_param)
             else:
                 raise ValueError('Invalid type for artifact_id: {}'.format(artifact.__class__.__name__))
+        elif artifact.meta.prop(MetaNames.FRAMEWORK_NAME).startswith("hybrid"):
+            return self._save_hybird_pipeline_model(artifact, query_param)
         else:
             return self._save_spark_pipeline_model(artifact, query_param)
 
@@ -245,6 +247,37 @@ class ModelCollection:
             print(traceback.format_exc())
             raise e
 
+    def _save_hybird_pipeline_model(self, artifact, query_param=None):
+        """
+        Saves model in repository service.
+
+        :param ScikitPipelineModelArtifact artifact: artifact to be saved in the repository service
+        :return: saved artifact with changed MetaProps
+        :rtype: ScikitPipelineModelArtifact
+        """
+        logger.debug('Creating a new hybrid model: {}'.format(artifact.name))
+
+        if not issubclass(type(artifact), ModelArtifact):
+            raise ValueError('Invalid type for artifact: {}'.format(artifact.__class__.__name__))
+
+        if artifact.meta.prop(MetaNames.MODEL_VERSION_URL) is not None:
+            raise ApiException(400, 'Invalid operation: save the same model artifact twice')
+
+        try:
+            if artifact.uid is None:
+                model_artifact = self._create_pipeline_model(artifact, query_param)
+            else:
+                model_artifact = artifact
+
+            if model_artifact.uid is None:
+                raise RuntimeError('Internal Error: Model without ID')
+            else:
+                return model_artifact
+        except Exception as e:
+            logger.info('Error in hybrid model creation')
+            import traceback
+            print(traceback.format_exc())
+
 
     def _save_tensorflow_pipeline_model(self, artifact, query_param=None):
         """
@@ -475,10 +508,8 @@ class ModelCollection:
 
     def _create_pipeline_model(self, model_artifact, query_param=None):
         model_input = self._prepare_model_input(model_artifact)
-
         model_output = self.repository_api.ml_assets_model_creation(model_input)
 
-
         if model_output is not None:
             location = model_output.metadata.url
             if location is not None:
@@ -500,11 +531,13 @@ class ModelCollection:
                         return new_async_artifact
                     else:
                         return new_artifact
+
+                else:
+                    return model_artifact_with_version
         else:
             logger.info('Location of the new pipeline model not found')
             raise ApiException(404, 'No artifact location')
 
-
     def _create_pipeline_model_version(self, model_artifact, query_param=None):
         model_version_input = self._get_version_input(model_artifact)
         r = self.repository_api.repository_model_version_creation(model_artifact.uid, model_version_input)
@@ -607,7 +640,7 @@ class ModelCollection:
                 ),
                 ModelVersionOutputEntity(
                     ModelVersionOutputEntityModel(model_output.metadata.guid, model_output.metadata.href),
-                    latest_version.content_href, None, None, content_status, content_location,hyper_parameters
+                    latest_version.content_href, None, None, content_status, content_location,hyper_parameters,category
                 )
             )
         else:
@@ -718,6 +751,16 @@ class ModelCollection:
         training_data_list = []
         model_input_evaluation = None
         content_location = None
+        category = None
+        label_column=artifact.meta.prop(MetaNames.LABEL_FIELD)
+        category = artifact.meta.prop(MetaNames.CATEGORY)
+
+        if artifact.meta.prop(MetaNames.CONTENT_LOCATION) is not None:
+            contentloc=json.loads(artifact.meta.prop(MetaNames.CONTENT_LOCATION))
+            content_location = ContentLocation(
+                contentloc.get('url', None),
+                contentloc.get('connection', None),
+                contentloc.get('source', None))
 
         if artifact.meta.prop(MetaNames.FRAMEWORK_NAME).startswith("scikit") \
                 or artifact.meta.prop(MetaNames.FRAMEWORK_NAME).startswith("xgboost"):
@@ -727,7 +770,7 @@ class ModelCollection:
                 name=artifact.name,
                 description=meta.prop(MetaNames.DESCRIPTION),
                 author=mauthor,
-                label_column=meta.prop(MetaNames.LABEL_FIELD),
+                label_column=label_column,
                 training_data_schema=meta.prop(MetaNames.TRAINING_DATA_SCHEMA),
                 hyper_parameters=hyper_param_list,
                 output_data_schema=output_data_schema,
@@ -746,6 +789,20 @@ class ModelCollection:
                 output_data_schema=output_data_schema ,
                 runtime_url=runtime_url_input
             )
+        elif artifact.meta.prop(MetaNames.FRAMEWORK_NAME).startswith("hybrid"):
+            return MlAssetsCreateModelInput(
+                tags=tags_data_list,
+                framework=mframework,
+                name=artifact.name,
+                description=meta.prop(MetaNames.DESCRIPTION),
+                author=mauthor,
+                label_column=label_column,
+                content_location= content_location,
+                hyper_parameters=hyper_param_list,
+                output_data_schema=output_data_schema,
+                runtime_url=runtime_url_input,
+                category=category
+            )
         else:
             if artifact.meta.prop(MetaNames.TRAINING_DATA_REFERENCE) is not None:
                 dataref_list=artifact.meta.prop(MetaNames.TRAINING_DATA_REFERENCE)
@@ -783,12 +840,6 @@ class ModelCollection:
                     artifact.meta.prop(MetaNames.EVALUATION_METHOD),
                     metrics)
 
-            if artifact.meta.prop(MetaNames.CONTENT_LOCATION) is not None:
-                contentloc=json.loads(artifact.meta.prop(MetaNames.CONTENT_LOCATION))
-                content_location = ContentLocation(
-                    contentloc.get('url', None),
-                    contentloc.get('connection', None),
-                    contentloc.get('source', None))
             if artifact.pipeline_artifact() is not None:
                 pipeline_artifact = artifact.pipeline_artifact()
                 model_input = MlAssetsCreateModelInput(
