@@ -10,16 +10,17 @@
 from __future__ import print_function
 from repository_v3.mlrepository import MetaNames
 from tabulate import tabulate
-import json
+import copy
 from watson_machine_learning_client.wml_resource import WMLResource
 from watson_machine_learning_client.utils import STR_TYPE, STR_TYPE_NAME
 from watson_machine_learning_client.log_util import get_logger
+from watson_machine_learning_client.wml_client_error import WMLClientError
 
 logger = get_logger('watson_machine_learning_client.metanames')
 
 
 class MetaProp:
-    def __init__(self, name, key, prop_type, required, example_value, ignored=False, hidden=False, default_value=''):
+    def __init__(self, name, key, prop_type, required, example_value, ignored=False, hidden=False, default_value='', path=None, transform=lambda x, client: x):
         self.key = key
         self.name = name
         self.prop_type = prop_type
@@ -28,16 +29,14 @@ class MetaProp:
         self.ignored = ignored
         self.hidden = hidden
         self.default_value = default_value
+        self.path = path if path is not None else '/' + key
+        self.transform = transform
 
 
 class MetaNamesBase:
     def __init__(self, meta_props_definitions):
         self._meta_props_definitions = meta_props_definitions
 
-        # if self.__class__.__name__ is not MetaNamesBase.__name__:
-        #     for meta_prop in self._meta_props_definitions:
-        #         meta_prop.name = list(self.__class__.__dict__.keys())[list(self.__class__.__dict__.values()).index(meta_prop.key)]
-
     def _validate(self, meta_props):
         for meta_prop in self._meta_props_definitions:
             if meta_prop.ignored is False:
@@ -109,6 +108,106 @@ Available MetaNames:
     def get_example_values(self):
         return dict((x.key, x.example_value) for x in filter(lambda x: not x.ignored and not x.hidden, self._meta_props_definitions))
 
+    def _generate_resource_metadata(self, meta_props, client=None, with_validation=False, initial_metadata={}):
+        if with_validation:
+            self._validate(meta_props)
+
+        metadata = copy.deepcopy(initial_metadata)
+
+        def update_map(m, path, el):
+            if type(m) is dict:
+                if len(path) == 1:
+                    m[path[0]] = el
+                else:
+                    if path[0] not in m:
+                        if type(path[1]) is not int:
+                            m[path[0]] = {}
+                        else:
+                            m[path[0]] = []
+                    update_map(m[path[0]], path[1:], el)
+            elif type(m) is list:
+                if len(path) == 1:
+                    if len(m) > len(path):
+                        m[path[0]] = el
+                    else:
+                        m.append(el)
+                else:
+                    if len(m) <= path[0]:
+                        m.append({})
+                    update_map(m[path[0]], path[1:], el)
+            else:
+                raise WMLClientError('Unexpected metadata path type: {}'.format(type(m)))
+
+
+        for meta_prop_def in filter(lambda x: not x.ignored, self._meta_props_definitions):
+            if meta_prop_def.key in meta_props:
+
+                path = [int(p) if p.isdigit() else p for p in meta_prop_def.path.split('/')[1:]]
+
+                update_map(
+                    metadata,
+                    path,
+                    meta_prop_def.transform(meta_props[meta_prop_def.key], client)
+                )
+
+        return metadata
+
+    def _generate_patch_payload(self, current_metadata, meta_props, client=None, with_validation=False):
+        if with_validation:
+            self._check_types_only(meta_props)
+
+        updated_metadata = self._generate_resource_metadata(meta_props, client, False, current_metadata)
+
+        patch_payload = []
+
+        def contained_path(metadata, path):
+            if path[0] in metadata:
+                if len(path) == 1:
+                    return [path[0]]
+                else:
+                    rest_of_path = contained_path(metadata[path[0]], path[1:])
+                    if rest_of_path is None:
+                        return [path[0]]
+                    else:
+                        return [path[0]] + rest_of_path
+            else:
+                return []
+
+        def get_value(metadata, path):
+            if len(path) == 1:
+                return metadata[path[0]]
+            else:
+                return get_value(metadata[path[0]], path[1:])
+
+        def already_in_payload(path):
+            return any([el['path'] == path for el in patch_payload])
+
+        def update_payload(path):
+            existing_path = contained_path(current_metadata, path)
+
+            if len(existing_path) == len(path):
+                patch_payload.append({
+                    'op': 'replace',
+                    'path': '/' + '/'.join(existing_path),
+                    'value': get_value(updated_metadata, existing_path)
+                })
+            else:
+                if not already_in_payload(existing_path):
+                    patch_payload.append({
+                        'op': 'add',
+                        'path': '/' + '/'.join(existing_path + [path[len(existing_path)]]),
+                        'value': get_value(updated_metadata, existing_path + [path[len(existing_path)]])
+                    })
+
+        for meta_prop_def in filter(lambda x: not x.ignored, self._meta_props_definitions):
+            if meta_prop_def.key in meta_props:
+
+                path = [int(p) if p.isdigit() else p for p in meta_prop_def.path.split('/')[1:]]
+
+                update_payload(path)
+
+        return patch_payload
+
 
 class TrainingConfigurationMetaNames(MetaNamesBase):
     _COMPUTE_CONFIGURATION_DEFAULT = 'k80'
@@ -151,13 +250,13 @@ class ExperimentMetaNames(MetaNamesBase):
     TRAINING_RESULTS_REFERENCE = "training_results_reference"
 
     _meta_props_definitions = [
-        MetaProp('NAME',                        NAME,                        STR_TYPE,   True,     u'Hand-written Digit Recognitionu'),
-        MetaProp('DESCRIPTION',                 DESCRIPTION,                 STR_TYPE,   False,    u'Hand-written Digit Recognition training'),
+        MetaProp('NAME',                        NAME,                        STR_TYPE,   True,     u'Hand-written Digit Recognitionu', path="/settings/name"),
+        MetaProp('DESCRIPTION',                 DESCRIPTION,                 STR_TYPE,   False,    u'Hand-written Digit Recognition training', path="/settings/description"),
         MetaProp('TAGS',                        TAGS,                        list,       False,    [{u'value': 'dsx-project.<project-guid>',u'description': 'DSX project guid'}]),
-        MetaProp('AUTHOR_NAME',                 AUTHOR_NAME,                 STR_TYPE,   False,    u'John Smith'),
+        MetaProp('AUTHOR_NAME',                 AUTHOR_NAME,                 STR_TYPE,   False,    u'John Smith', path="/settings/author/name"),
         MetaProp('AUTHOR_EMAIL',                AUTHOR_EMAIL,                STR_TYPE,   False,    u'john.smith@x.com', ignored=True),
-        MetaProp('EVALUATION_METHOD',           EVALUATION_METHOD,           STR_TYPE,   False,    u'multiclass'),
-        MetaProp('EVALUATION_METRICS',          EVALUATION_METRICS,          list,       False,    [u'accuracy']),
+        MetaProp('EVALUATION_METHOD',           EVALUATION_METHOD,           STR_TYPE,   False,    u'multiclass', path="/settings/evaluation_definition/method"),
+        MetaProp('EVALUATION_METRICS',          EVALUATION_METRICS,          list,       False,    [u'accuracy'], path="/settings/evaluation_definition/metrics", transform=lambda m, client: [{u'name': x} for x in m]),
         MetaProp('TRAINING_REFERENCES',         TRAINING_REFERENCES,         list,       True,     [{u'training_definition_url': u'https://ibm-watson-ml.mybluemix.net/v3/ml_assets/training_definitions/12345',u'compute_configuration': {u'name': TrainingConfigurationMetaNames._COMPUTE_CONFIGURATION_DEFAULT}},{u'training_definition_url': u'https://ibm-watson-ml.mybluemix.net/v3/ml_assets/training_definitions/67890'}]),
         MetaProp('TRAINING_DATA_REFERENCE',     TRAINING_DATA_REFERENCE,     dict,       True,     {u'connection': {u'endpoint_url': u'https://s3-api.us-geo.objectstorage.softlayer.net',u'access_key_id': u'***',u'secret_access_key': u'***'},u'source': {u'bucket': u'train-data'},u'type': u's3'}),
         MetaProp('TRAINING_RESULTS_REFERENCE',  TRAINING_RESULTS_REFERENCE,  dict,       True,     {u'connection': {u'endpoint_url': u'https://s3-api.us-geo.objectstorage.softlayer.net',u'access_key_id': u'***',u'secret_access_key': u'***'},u'target': {u'bucket': u'result-data'},u'type': 's3'})
@@ -183,12 +282,12 @@ class ModelDefinitionMetaNames(MetaNamesBase):
     _meta_props_definitions = [
         MetaProp('NAME',                NAME, STR_TYPE, True, u'my_training_definition'),
         MetaProp('DESCRIPTION',         DESCRIPTION, STR_TYPE, False, u'my_description'),
-        MetaProp('AUTHOR_NAME',         AUTHOR_NAME, STR_TYPE, False, u'John Smith'),
+        MetaProp('AUTHOR_NAME',         AUTHOR_NAME, STR_TYPE, False, u'John Smith', path='/author/name'),
         MetaProp('AUTHOR_EMAIL',        AUTHOR_EMAIL, STR_TYPE, False, u'john.smith@x.com', ignored=True),
-        MetaProp('FRAMEWORK_NAME',      FRAMEWORK_NAME, STR_TYPE, True, u'tensorflow'),
-        MetaProp('FRAMEWORK_VERSION',   FRAMEWORK_VERSION, STR_TYPE, True, u'1.5'),
-        MetaProp('RUNTIME_NAME',        RUNTIME_NAME, STR_TYPE, True, u'python'),
-        MetaProp('RUNTIME_VERSION',     RUNTIME_VERSION, STR_TYPE, True, u'3.5'),
+        MetaProp('FRAMEWORK_NAME',      FRAMEWORK_NAME, STR_TYPE, True, u'tensorflow', path='/framework/name'),
+        MetaProp('FRAMEWORK_VERSION',   FRAMEWORK_VERSION, STR_TYPE, True, u'1.5', path='/framework/version'),
+        MetaProp('RUNTIME_NAME',        RUNTIME_NAME, STR_TYPE, True, u'python', path='/framework/runtimes/0/name'),
+        MetaProp('RUNTIME_VERSION',     RUNTIME_VERSION, STR_TYPE, True, u'3.5', path='/framework/runtimes/0/version'),
         MetaProp('EXECUTION_COMMAND',   EXECUTION_COMMAND, STR_TYPE, True, u'python3 tensorflow_mnist_softmax.py --trainingIters 20')
     ]
 
@@ -215,24 +314,26 @@ class ModelMetaNames(MetaNamesBase):
     LABEL_FIELD = MetaNames.LABEL_FIELD
     TRANSFORMED_LABEL_FIELD = MetaNames.TRANSFORMED_LABEL_FIELD
     RUNTIME_UID = "runtime_uid"
+    TRAINING_DEFINITION_URL = MetaNames.TRAINING_DEFINITION_URL
 
     _meta_props_definitions = [
         MetaProp('NAME',                    NAME,                        STR_TYPE,   True,   "my_model"),
-        MetaProp('DESCRIPTION',             DESCRIPTION,                 STR_TYPE,   False,  "my_description"),
-        MetaProp('AUTHOR_NAME',             AUTHOR_NAME,                 STR_TYPE,   False,  u'John Smith'),
+        MetaProp('DESCRIPTION',             DESCRIPTION,                 STR_TYPE,   False,  "my_description", path="/description"),
+        MetaProp('AUTHOR_NAME',             AUTHOR_NAME,                 STR_TYPE,   False,  u'John Smith', path="/author/name"),
         MetaProp('AUTHOR_EMAIL',            AUTHOR_EMAIL,                STR_TYPE,   False,  u'john.smith@x.com', ignored=True),
-        MetaProp('FRAMEWORK_NAME',          FRAMEWORK_NAME,              STR_TYPE,   False,  u'tensorflow'),
-        MetaProp('FRAMEWORK_VERSION',       FRAMEWORK_VERSION,           STR_TYPE,   False,  u'1.5'),
-        MetaProp('FRAMEWORK_LIBRARIES',     FRAMEWORK_LIBRARIES,         list,       False,  [{'name': 'keras', 'version': '2.1.3'}]),
-        MetaProp('RUNTIME_NAME',            RUNTIME_NAME,                STR_TYPE,   False,  u'python'),
-        MetaProp('RUNTIME_VERSION',         RUNTIME_VERSION,             STR_TYPE,   False,  u'3.5'),
-        MetaProp('TRAINING_DATA_REFERENCE', TRAINING_DATA_REFERENCE,     dict,       False,  {}),
-        MetaProp('EVALUATION_METHOD',       EVALUATION_METHOD,           STR_TYPE,   False,  "multiclass"),
-        MetaProp('EVALUATION_METRICS',      EVALUATION_METRICS,          list,       False,  [{"name": "accuracy","value": 0.64,"threshold": 0.8}]),
-        MetaProp('OUTPUT_DATA_SCHEMA',      OUTPUT_DATA_SCHEMA,          dict,       False,  {'fields': [{'metadata': {}, 'type': 'string', 'name': 'GENDER', 'nullable': True}, {'metadata': {'modeling_role': 'transformed-target'}, 'type': 'double', 'name': 'PRODUCT_LINE_IX', 'nullable': False}], 'type': 'struct'}),
-        MetaProp('LABEL_FIELD',             LABEL_FIELD,                 STR_TYPE,   False,  'PRODUCT_LINE'),
-        MetaProp('TRANSFORMED_LABEL_FIELD', TRANSFORMED_LABEL_FIELD,     STR_TYPE,   False,  'PRODUCT_LINE_IX'),
-        MetaProp('RUNTIME_UID',             RUNTIME_UID,                 STR_TYPE,   False,  '53628d69-ced9-4f43-a8cd-9954344039a8')
+        MetaProp('FRAMEWORK_NAME',          FRAMEWORK_NAME,              STR_TYPE,   False,  u'tensorflow', path="/framework/name"),
+        MetaProp('FRAMEWORK_VERSION',       FRAMEWORK_VERSION,           STR_TYPE,   False,  u'1.5', path="/framework/version"),
+        MetaProp('FRAMEWORK_LIBRARIES',     FRAMEWORK_LIBRARIES,         list,       False,  [{'name': 'keras', 'version': '2.1.3'}], path="/framework/libraries"),
+        MetaProp('RUNTIME_NAME',            RUNTIME_NAME,                STR_TYPE,   False,  u'python', path="/framework/runtimes/0/name"),
+        MetaProp('RUNTIME_VERSION',         RUNTIME_VERSION,             STR_TYPE,   False,  u'3.5', path="/framework/runtimes/0/version"),
+        MetaProp('TRAINING_DATA_REFERENCE', TRAINING_DATA_REFERENCE,     dict,       False,  {}, path="/training_data_reference"),
+        MetaProp('EVALUATION_METHOD',       EVALUATION_METHOD,           STR_TYPE,   False,  "multiclass", path="/evaluation/method"),
+        MetaProp('EVALUATION_METRICS',      EVALUATION_METRICS,          list,       False,  [{"name": "accuracy","value": 0.64,"threshold": 0.8}], path="/evaluation/metrics"),
+        MetaProp('OUTPUT_DATA_SCHEMA',      OUTPUT_DATA_SCHEMA,          dict,       False,  {'fields': [{'metadata': {}, 'type': 'string', 'name': 'GENDER', 'nullable': True}, {'metadata': {'modeling_role': 'transformed-target'}, 'type': 'double', 'name': 'PRODUCT_LINE_IX', 'nullable': False}], 'type': 'struct'}, path="/output_data_schema"),
+        MetaProp('LABEL_FIELD',             LABEL_FIELD,                 STR_TYPE,   False,  'PRODUCT_LINE', path="/label_column"),
+        MetaProp('TRANSFORMED_LABEL_FIELD', TRANSFORMED_LABEL_FIELD,     STR_TYPE,   False,  'PRODUCT_LINE_IX', path="/transformed_label"),
+        MetaProp('RUNTIME_UID',             RUNTIME_UID,                 STR_TYPE,   False,  '53628d69-ced9-4f43-a8cd-9954344039a8', path="/runtime_url", transform=lambda x, client: client.repository._href_definitions.get_runtime_href(x)),
+        MetaProp('TRAINING_DEFINITION_URL', TRAINING_DEFINITION_URL,     STR_TYPE,   False,  'https://us-south.ml.cloud.ibm.com/v3/ml_assets/training_definitions/ff24ebec-86af-4f2d-ab4c-2fc9e2a3efbc')
     ]
 
     __doc__ = MetaNamesBase(_meta_props_definitions)._generate_doc('models')
@@ -290,7 +391,7 @@ class FunctionMetaNames(MetaNamesBase):
     _meta_props_definitions = [
         MetaProp('NAME',                NAME,                STR_TYPE,   True,   "ai_function"),
         MetaProp('DESCRIPTION',         DESCRIPTION,         STR_TYPE,   False,  "This is ai function"),
-        MetaProp('RUNTIME_UID',         RUNTIME_UID,         STR_TYPE,   False,  '53628d69-ced9-4f43-a8cd-9954344039a8'),
+        MetaProp('RUNTIME_UID',         RUNTIME_UID,         STR_TYPE,   False,  '53628d69-ced9-4f43-a8cd-9954344039a8', path="/runtime_url", transform=lambda x, client: client.repository._href_definitions.get_runtime_href(x)),
         MetaProp('INPUT_DATA_SCHEMA',   INPUT_DATA_SCHEMA,   dict,       False,  {"type": "struct", "fields": [{"name": "x", "type": "double", "nullable": False, "metadata": {}}, {"name": "y", "type": "double", "nullable": False, "metadata": {}}]}),
         MetaProp('OUTPUT_DATA_SCHEMA',  OUTPUT_DATA_SCHEMA,  dict,       False,  {"type": "struct", "fields": [{"name": "multiplication", "type": "double", "nullable": False, "metadata": {}}]}),
         MetaProp('TAGS',                TAGS,                list,       False,  [{"value": "ProjectA", "description": "Functions created for ProjectA"}])
