@@ -205,8 +205,8 @@ class Training(WMLResource):
         >>>  client.training.ConfigurationMetaNames.TRAINING_DATA_REFERENCE: {
         >>>          u'connection': {
         >>>              u'endpoint_url': u'https://s3-api.us-geo.objectstorage.service.networklayer.com',
-        >>>              u'aws_access_key_id': u'***',
-        >>>              u'aws_secret_access_key': u'***'
+        >>>              u'access_key_id': u'***',
+        >>>              u'secret_access_key': u'***'
         >>>          },
         >>>          u'source': {
         >>>              u'bucket': u'wml-dev',
@@ -216,14 +216,16 @@ class Training(WMLResource):
         >>> client.training.ConfigurationMetaNames.TRAINING_RESULTS_REFERENCE: {
         >>>          u'connection': {
         >>>              u'endpoint_url': u'https://s3-api.us-geo.objectstorage.service.networklayer.com',
-        >>>              u'aws_access_key_id': u'***',
-        >>>              u'aws_secret_access_key': u'***'
+        >>>              u'access_key_id': u'***',
+        >>>              u'secret_access_key': u'***'
         >>>          },
         >>>          u'target': {
         >>>              u'bucket': u'wml-dev-results',
         >>>          }
         >>>          u'type': u's3'
         >>>      },
+        >>> client.training.ConfigurationMetaNames.COMPUTE_CONFIGURATION: {'name': 'p100'},
+        >>> client.training.ConfigurationMetaNames.EXECUTION_COMMAND: "python3 tensorflow_mnist_softmax.py --trainingIters 20",
         >>> }
         >>> run_details = client.training.run(definition_uid, meta_props=metadata)
         >>> run_uid = client.training.get_run_uid(run_details)
